#ifndef _XsmGridP_h
#define _XsmGridP_h

/* include the public header as well as the super class private header */

#include "Grid.h"
#include <Xm/ManagerP.h>

/*--------------------------------------------------------------------------*/
/*************************
 * THE GRID CLASS RECORD *
 *************************/

/* Define the types of procedure pointers not defined in the Intrinsics. */

typedef void (*XsmUpdateGridProc)(Widget, Dimension *, Dimension *) ;
typedef void (*XsmAddChildProc)(Widget, Widget, Dimension, Dimension) ;

/* Define inheritance macros for all knob class procedures. */

#define XsmInheritUpdateGridDimensions ((XsmUpdateGridProc) _XtInherit)
#define XsmInheritAddChild ((XsmAddChildProc) _XtInherit)
#define XsmInheritLayoutChildren ((XtWidgetProc) _XtInherit)
#define XsmInheritDrawGrid ((XtWidgetProc) _XtInherit)
#define XsmInheritEraseGrid ((XtWidgetProc) _XtInherit)

/* The Grid widget's part of the widget class record. */

typedef struct {
        XsmUpdateGridProc       update_grid;     /* update grid dimensions */
        XsmAddChildProc		add_child;       /* Add child to grid */
        XtWidgetProc            layout_children; /* Layout widget children */
        XtWidgetProc            draw_grid;       /* Draw grid lines */
        XtWidgetProc            erase_grid;      /* Erase grid lines */
        XtPointer               extension;       /* Pointer to extension
                                                    record */
} XsmGridClassPart;

/* All of the widget class parts together form */
/* the widget class record.                    */

typedef struct _XsmGridClassRec {
   CoreClassPart core_class;
   CompositeClassPart  composite_class;
   ConstraintClassPart constraint_class;
   XmManagerClassPart  manager_class;
   XsmGridClassPart grid_class;
} XsmGridClassRec;


/*--------------------------------------------------------------------------*/
/****************************
 * THE GRID INSTANCE RECORD *
 ****************************/

/* This Grid widget's instance record.  It includes resource */
/* and internal variables.                                     */

typedef struct _XsmGridPart {
   Dimension columns;
   Dimension rows;
   Dimension margin_width;
   Dimension margin_height;
   Dimension orig_margin_width;
   Dimension orig_margin_height;
   Dimension h_spacing;
   Dimension v_spacing;
   WidgetList * widget_grid;
   Dimension * row_height;
   Dimension * col_width;
   XtEnum horiz_alignment;
   XtEnum vert_alignment;
   Boolean show_grid_lines;
   Boolean lines_visible;
   GC gc;
} XsmGridPart;

/* All of the widget instance parts together form */
/* the widget instance record.                    */

typedef struct _XsmGridRec {
    CorePart    core;
    CompositePart  composite;
    ConstraintPart constraint;
    XmManagerPart  manager;
    XsmGridPart grid;
} XsmGridRec;

/*--------------------------------------------------------------------------*/
/***************************************
 * THE GRID CONSTRAINT INSTANCE RECORD *
 ***************************************/

/* Used only with constraint managers, the constraint part */
/* contains resources and internal variables that are to   */
/* be placed on each instance of children of this manager  */
/* widget.                                                 */

typedef struct _XsmGridConstraintPart
{
    Dimension row_pos;
    Dimension col_pos;
    Dimension orig_row_pos;
    Dimension orig_col_pos;
    Boolean auto_placed;
} XsmGridConstraintPart;

/* Only the active constraint parts of widget are        */
/* added here to form the constraint record. Neither     */
/* Core, Composite, or Constraint have active constraint */
/* parts.  Manager does have an active constraint part   */
/* so it must be added for constraint widgets.  Not      */
/* all manager are constraint widgets.                   */

typedef struct _XsmGridConstraintRec
{
   XmManagerConstraintPart manager;
   XsmGridConstraintPart grid;
} XsmGridConstraintRec, * XsmGridConstraintPtr;

#endif /* _XsmGridP_h */
/* DON't ADD STUFF AFTER THIS #endif */
